from dataclasses import dataclass


@dataclass
class GeneralSchemaEnhancer:
    """
    Main functions have moved to core linkml, see https://github.com/linkml/linkml/pull/854

    This is currently a stub for future enhancements
    """
    pass






